#ifndef   TEXCOORDP
#define   TEXCOORDP highp
#endif // TEXCOORDP

#ifndef   CONFIG_SHADER_FRAMEBUFFER_FETCH
#define   CONFIG_SHADER_FRAMEBUFFER_FETCH 0
#endif // CONFIG_SHADER_FRAMEBUFFER_FETCH

#ifndef   IS_EXTERNAL_TEXTURE
#define   IS_EXTERNAL_TEXTURE 0
#endif // IS_EXTERNAL_TEXTURE

#if       IS_EXTERNAL_TEXTURE
#extension GL_OES_EGL_image_external: require
#endif // IS_EXTERNAL_TEXTURE


varying TEXCOORDP vec2 vTexCoord;

#ifdef RETROVK_VERTEX_SHADER

precision highp float;

attribute vec2 aPosition;
attribute vec2 aTexCoord;

uniform mat4 uImageMatrix;
uniform mat4 uMVP;


void main()
{
  gl_Position = uMVP * vec4(aPosition, 0, 1);

  vTexCoord = (uImageMatrix * vec4(aTexCoord, 0, 1)).xy;
}

#endif // RETROVK_VERTEX_SHADER

#ifdef RETROVK_FRAGMENT_SHADER

#ifndef   TILE_EDGE_LENGTH
#define   TILE_EDGE_LENGTH 64
#endif // TILE_EDGE_LENGTH

#ifndef   GRID_COLS
#define   GRID_COLS 8
#endif // GRID_COLS

#ifndef   GRID_ROWS
#define   GRID_ROWS 8
#endif // GRID_ROWS

precision mediump float;

#if       CONFIG_SHADER_FRAMEBUFFER_FETCH
// no backdrop
#else  // CONFIG_SHADER_FRAMEBUFFER_FETCH

#if       IS_EXTERNAL_TEXTURE
uniform samplerExternalOES sImage;
#else  // IS_EXTERNAL_TEXTURE
uniform sampler2D sImage;
#endif // IS_EXTERNAL_TEXTURE

#endif // CONFIG_SHADER_FRAMEBUFFER_FETCH

uniform sampler2D sColorPalette;

uniform float uColorPaletteWeight;

vec2 GetTileLeftBottom(float tile_index) {
    float grid_y = floor(tile_index / float(GRID_COLS));
    float grid_x = tile_index - grid_y * float(GRID_COLS);

    return vec2(
        grid_x / float(GRID_COLS),
        grid_y / float(GRID_ROWS)
    );
}

vec3 ColorPalette3D(vec3 color, float weight) {
  float tile_left   = 0.5 / float(GRID_COLS * TILE_EDGE_LENGTH);
  float tile_bottom = 0.5 / float(GRID_ROWS * TILE_EDGE_LENGTH);
  float tile_right  = (float(TILE_EDGE_LENGTH) - 0.5) / float(GRID_COLS * TILE_EDGE_LENGTH);
  float tile_top    = (float(TILE_EDGE_LENGTH) - 0.5) / float(GRID_ROWS * TILE_EDGE_LENGTH);

  vec2 position_in_tile = mix(
      vec2(tile_left, tile_bottom),
      vec2(tile_right, tile_top),
      color.rg
  );

  float tile_index = color.b * float(TILE_EDGE_LENGTH - 1);

  vec2 tile_lb_0 = GetTileLeftBottom(floor(tile_index));
  vec2 tile_lb_1 = GetTileLeftBottom(ceil(tile_index));

  vec2 position_0 = tile_lb_0 + position_in_tile;
  vec2 position_1 = tile_lb_1 + position_in_tile;

  vec3 new_color_0 = texture2D(sColorPalette, vec2(position_0.x, 1.0 - position_0.y)).rgb;
  vec3 new_color_1 = texture2D(sColorPalette, vec2(position_1.x, 1.0 - position_1.y)).rgb;
  vec3 new_color = mix(new_color_0, new_color_1, fract(tile_index));

  return mix(color, new_color, weight);
}

void main() {

#if       CONFIG_SHADER_FRAMEBUFFER_FETCH

#if defined(GL_EXT_shader_framebuffer_fetch)
  vec4 color = gl_LastFragData[0];
#elif defined(GL_ARM_shader_framebuffer_fetch)
  vec4 color = gl_LastFragColorARM;
#endif

#else  // CONFIG_SHADER_FRAMEBUFFER_FETCH
  vec4 color = texture2D(sImage, vTexCoord);
#endif // CONFIG_SHADER_FRAMEBUFFER_FETCH

  vec3 new_color = ColorPalette3D(color.rgb, uColorPaletteWeight);

  gl_FragColor = vec4(new_color, 1);
}

#endif // RETROVK_FRAGMENT_SHADER
