#ifndef   TEXCOORDP
#define   TEXCOORDP highp
#endif // TEXCOORDP

#ifndef   IS_EXTERNAL_TEXTURE
#define   IS_EXTERNAL_TEXTURE 0
#endif // IS_EXTERNAL_TEXTURE

#if       IS_EXTERNAL_TEXTURE
#extension GL_OES_EGL_image_external: require
#define SAMPLER_TEXTURE_COLOR_IN samplerExternalOES
#else  // IS_EXTERNAL_TEXTURE
#define SAMPLER_TEXTURE_COLOR_IN sampler2D
#endif // IS_EXTERNAL_TEXTURE

varying TEXCOORDP vec2 vTexCoord;
varying TEXCOORDP vec2 vTexCoordIn;


#ifdef RETROVK_VERTEX_SHADER

precision highp float;

attribute vec4 aPosition;
attribute vec2 aTexCoord;

uniform mat4 uImageMatrix;

void main()
{
  gl_Position = aPosition;

  vTexCoord = aTexCoord;
  vTexCoordIn = (uImageMatrix * vec4(aTexCoord, 0, 1)).xy;
}

#endif // RETROVK_VERTEX_SHADER


#ifdef RETROVK_FRAGMENT_SHADER

precision mediump float;

uniform sampler2D sColorMean;
uniform sampler2D sColorVariance;
uniform SAMPLER_TEXTURE_COLOR_IN sImage;
uniform sampler2D sColorPalette;

uniform vec4 uBlendSharpenLighten;
uniform vec2 uTexelOffset;


#define TILE_COUNT_PER_ROW 8
#define TILE_COUNT_PER_COL 8
#define TILE_EDGE_LENGTH 64

vec2 GetOffsetInTile(vec2 color) {
    float tileCountPerRow = float(TILE_COUNT_PER_ROW);
    float tileCountPerCol = float(TILE_COUNT_PER_COL);
    float tileEdgeLength = float(TILE_EDGE_LENGTH);

    float startX = 0.5 / tileEdgeLength / tileCountPerRow;
    float endX = (1.0 - 0.5 / tileEdgeLength) / tileCountPerRow;

    float startY = -0.5 / tileEdgeLength / tileCountPerCol;
    float endY = -(1.0 - 0.5 / tileEdgeLength) / tileCountPerCol;

    return mix(vec2(startX, startY), vec2(endX, endY), color);
}

vec3 GetColor2D(float tileIndex, vec2 offset) {
    float tileCountPerRow = float(TILE_COUNT_PER_ROW);
    float tileCountPerCol = float(TILE_COUNT_PER_COL);

    float tileIndexY = floor(tileIndex / tileCountPerRow);
    float tileIndexX = tileIndex - (tileIndexY * tileCountPerRow);

    float tileX = tileIndexX / tileCountPerRow;
    float tileY = 1.0 - tileIndexY / tileCountPerCol;

    vec2 coord = vec2(tileX, tileY) + offset;
    return texture2D(sColorPalette, coord).rgb;
}

vec3 ColorPalette3D(vec3 color, float weight) {

    float span = float(TILE_EDGE_LENGTH);

#ifdef SAMPLER_3D_TEXTURE_COLOR_PALETTE
    float valMin = 0.5 / span;
    float valMax = (span - 0.5) / span;

    vec3 newColor = texture3D(sColorPalette, mix(vec3(valMin), vec3(valMax), color));
#else // SAMPLER_3D_TEXTURE_COLOR_PALETTE

    float tileIndex = color.b * (span - 1.0);

    vec2 offset = GetOffsetInTile(color.rg);

    vec3 newColor0 = GetColor2D(floor(tileIndex), offset);
    vec3 newColor1 = GetColor2D(ceil(tileIndex), offset);

    vec3 newColor = mix(newColor0, newColor1, fract(tileIndex));
#endif // SAMPLER_3D_TEXTURE_COLOR_PALETTE

    return mix(color, newColor, weight);
}

void main()
{
    vec3 varColor = texture2D(sColorVariance, vTexCoord).rgb;
    vec3 meanColor = texture2D(sColorMean, vTexCoord).rgb;
    vec3 iColor = texture2D(sImage, vTexCoordIn).rgb;
    float blurAlpha = uBlendSharpenLighten.x;
    float sharpenAlpha = uBlendSharpenLighten.y;
    float lightenFactor = uBlendSharpenLighten.z;

    float p = clamp((min(iColor.r, meanColor.r - 0.1) - 0.2) * 4.0, 0.0, 1.0);
    float meanVar = (varColor.r + varColor.g + varColor.b) / 3.0;
    float kMin = clamp((0.11 / (meanVar + 0.1) - meanVar * 0.4) * p * blurAlpha, 0.0, 1.0);
    vec3 resultColor;

    resultColor = clamp(mix(iColor, meanColor, kMin), 0.0, 1.0);
    resultColor = ColorPalette3D(resultColor, lightenFactor);

    gl_FragColor = vec4(resultColor, 1.0);
}

#endif // RETROVK_FRAGMENT_SHADER
