#ifndef   TEXCOORDP
#define   TEXCOORDP highp
#endif // TEXCOORDP

#ifdef RETROVK_VERTEX_SHADER
#define varying out
#endif

#ifdef RETROVK_FRAGMENT_SHADER
#define varying in
#endif

#ifndef   TIXEL_MAX_VARYING_VECTORS
#define   TIXEL_MAX_VARYING_VECTORS 8
#endif // TIXEL_MAX_VARYING_VECTORS

#ifndef   KERNEL_LENGTH
#define   KERNEL_LENGTH 9
#endif // KERNEL_LENGTH

#ifndef   IS_EXTERNAL_TEXTURE
#define   IS_EXTERNAL_TEXTURE 0
#endif // IS_EXTERNAL_TEXTURE

#if       IS_EXTERNAL_TEXTURE
#extension GL_OES_EGL_image_external: enable
#extension GL_OES_EGL_image_external_essl3: enable
#define SAMPLER_TEXTURE_COLOR_IN samplerExternalOES
#else //  IS_EXTERNAL_TEXTURE
#define SAMPLER_TEXTURE_COLOR_IN sampler2D
#endif // IS_EXTERNAL_TEXTURE

#if       (KERNEL_LENGTH / 2) > TIXEL_MAX_VARYING_VECTORS
#define TEX_COORD_ARRAY_LENGTH_0 TIXEL_MAX_VARYING_VECTORS
#else
#define TEX_COORD_ARRAY_LENGTH_0 (KERNEL_LENGTH / 2)
#endif

#if       (KERNEL_LENGTH - TEX_COORD_ARRAY_LENGTH_0) > (TIXEL_MAX_VARYING_VECTORS - 1)
#define TEX_COORD_ARRAY_LENGTH_1 (TIXEL_MAX_VARYING_VECTORS - 1)
#else
#define TEX_COORD_ARRAY_LENGTH_1 (KERNEL_LENGTH - TEX_COORD_ARRAY_LENGTH_0)
#endif

varying TEXCOORDP vec2 vTexCoord0[TEX_COORD_ARRAY_LENGTH_0];
varying TEXCOORDP vec2 vTexCoord1[TEX_COORD_ARRAY_LENGTH_1];
varying TEXCOORDP vec2 vTexCoord;


#ifdef RETROVK_VERTEX_SHADER

in vec4 aPosition;
in vec2 aTexCoord;

uniform vec2 uTexelOffset;

uniform mat4 uImageMatrix;

void main()
{
  gl_Position = aPosition;
  vTexCoord = (uImageMatrix * vec4(aTexCoord, 0, 1)).xy;

  vec2 texCoord = aTexCoord;
  vec2 singleStepOffset = uTexelOffset;

  const float start = -float(KERNEL_LENGTH) / 2.0 + 0.5;

  for (int i = 0; i < TEX_COORD_ARRAY_LENGTH_0; ++i) {
    float scale = start + float(i);
    vTexCoord0[i] = texCoord + singleStepOffset * scale;
  }
  for (int i = 0; i < TEX_COORD_ARRAY_LENGTH_1; ++i) {
    float scale = start + float(TEX_COORD_ARRAY_LENGTH_0 + i);
    vTexCoord1[i] = texCoord + singleStepOffset * scale;
  }
}

#endif // RETROVK_VERTEX_SHADER

#ifdef RETROVK_FRAGMENT_SHADER

precision mediump float;

uniform SAMPLER_TEXTURE_COLOR_IN    sImage;
uniform sampler2D sColorMean;

layout(location = 0) out vec4 cVariance;
layout(location = 1) out vec4 cMean;

void main() {
  vec3 sum = vec3(0);
  for (int i = 0; i < TEX_COORD_ARRAY_LENGTH_0; ++i) {
    sum += texture(sColorMean, vTexCoord0[i]).rgb;
  }
  for (int i = 0; i < TEX_COORD_ARRAY_LENGTH_1; ++i) {
    sum += texture(sColorMean, vTexCoord1[i]).rgb;
  }

  int dynamicStart = TEX_COORD_ARRAY_LENGTH_0 + TEX_COORD_ARRAY_LENGTH_1;
  vec2 dynamicStartTexCoord = vTexCoord1[TEX_COORD_ARRAY_LENGTH_1 - 1];
  vec2 stepOffset = dynamicStartTexCoord - vTexCoord1[TEX_COORD_ARRAY_LENGTH_1 - 2];
  for (int i = dynamicStart; i < KERNEL_LENGTH; ++i) {
    float scale = float(TEX_COORD_ARRAY_LENGTH_0 + i);
    vec2 texCoord = dynamicStartTexCoord + stepOffset * scale;
    sum += texture(sColorMean, texCoord).rgb;
  }

  vec3 meanColor = sum / float(KERNEL_LENGTH);
  cMean = vec4(meanColor, 1.0);

  vec3 iColor = texture(sImage, vTexCoord).rgb;

  vec3 diffColor = (iColor - meanColor) * 7.07;
  diffColor = min(diffColor * diffColor, 1.0);

  cVariance = vec4(diffColor, 1.0);
}

#endif // RETROVK_FRAGMENT_SHADER
