#ifndef   TEXCOORDP
#define   TEXCOORDP highp
#endif // TEXCOORDP

#ifdef RETROVK_VERTEX_SHADER
#define varying out
#define attribute in
#endif

#ifdef RETROVK_FRAGMENT_SHADER
#define varying in
#endif


#ifndef   IS_EXTERNAL_TEXTURE
#define   IS_EXTERNAL_TEXTURE 0
#endif // IS_EXTERNAL_TEXTURE

#if       IS_EXTERNAL_TEXTURE
#extension GL_OES_EGL_image_external: enable
#extension GL_OES_EGL_image_external_essl3: enable
#define SAMPLER_TEXTURE_COLOR_IN samplerExternalOES
#else  // IS_EXTERNAL_TEXTURE
#define SAMPLER_TEXTURE_COLOR_IN sampler2D
#endif // IS_EXTERNAL_TEXTURE

#define SHARPEN_TAP_COUNT 9

const vec2 uSharpenOffsetArray[SHARPEN_TAP_COUNT] =
    vec2[SHARPEN_TAP_COUNT] (
      vec2(0.000000, 0.000000),
      vec2(-1.182425, -1.182425),
      vec2(-1.182425, -1.817575),
      vec2(-1.817575, -1.182425),
      vec2(-1.817575, -1.817575),
      vec2(0.000000, -1.182425),
      vec2(0.000000, 1.182425),
      vec2(-1.182425, 0.000000),
      vec2(1.182425, 0.000000)
    );

const float uSharpenWeightArray[SHARPEN_TAP_COUNT] =
    float[SHARPEN_TAP_COUNT] (
      1.0 - 0.162103,
      -0.089216,
      -0.089216,
      -0.089216,
      -0.089216,
      -0.120259,
      -0.120259,
      -0.120259,
      -0.120259
    );


varying TEXCOORDP vec2 vTexCoord[SHARPEN_TAP_COUNT];
varying TEXCOORDP vec2 vTexCoordI;


#ifdef RETROVK_VERTEX_SHADER

precision highp float;

attribute vec4 aPosition;
attribute vec2 aTexCoord;

uniform mat4 uImageMatrix;
uniform vec2 uTexelOffset;

void main()
{
  gl_Position = aPosition;

  vTexCoordI = aTexCoord;
  for (int i = 0; i < SHARPEN_TAP_COUNT; ++i) {
    vec2 tex_coord = aTexCoord + uSharpenOffsetArray[i] * uTexelOffset;
    vTexCoord[i] = (uImageMatrix * vec4(tex_coord, 0, 1)).xy;
  }
}

#endif // RETROVK_VERTEX_SHADER


#ifdef RETROVK_FRAGMENT_SHADER

precision mediump float;

uniform sampler2D sColorMean;
uniform sampler2D sColorVariance;
uniform SAMPLER_TEXTURE_COLOR_IN sImage;
uniform sampler2D sColorPalette;

uniform vec4 uBlendSharpenLighten;
uniform vec2 uTexelOffset;

layout(location = 0) out vec4 cColor;

#define TILE_COUNT_PER_ROW 8
#define TILE_COUNT_PER_COL 8
#define TILE_EDGE_LENGTH 64

vec2 GetOffsetInTile(vec2 color) {
    float tileCountPerRow = float(TILE_COUNT_PER_ROW);
    float tileCountPerCol = float(TILE_COUNT_PER_COL);
    float tileEdgeLength = float(TILE_EDGE_LENGTH);

    float startX = 0.5 / tileEdgeLength / tileCountPerRow;
    float endX = (1.0 - 0.5 / tileEdgeLength) / tileCountPerRow;

    float startY = -0.5 / tileEdgeLength / tileCountPerCol;
    float endY = -(1.0 - 0.5 / tileEdgeLength) / tileCountPerCol;

    return mix(vec2(startX, startY), vec2(endX, endY), color);
}

vec3 GetColor2D(float tileIndex, vec2 offset) {
    float tileCountPerRow = float(TILE_COUNT_PER_ROW);
    float tileCountPerCol = float(TILE_COUNT_PER_COL);

    float tileIndexY = floor(tileIndex / tileCountPerRow);
    float tileIndexX = tileIndex - (tileIndexY * tileCountPerRow);

    float tileX = tileIndexX / tileCountPerRow;
    float tileY = 1.0 - tileIndexY / tileCountPerCol;

    vec2 coord = vec2(tileX, tileY) + offset;
    return texture(sColorPalette, coord).rgb;
}

vec3 ColorPalette3D(vec3 color, float weight) {

    float span = float(TILE_EDGE_LENGTH);

#ifdef SAMPLER_3D_TEXTURE_COLOR_PALETTE
    float valMin = 0.5 / span;
    float valMax = (span - 0.5) / span;

    vec3 newColor = texture3D(sColorPalette, mix(vec3(valMin), vec3(valMax), color));
#else // SAMPLER_3D_TEXTURE_COLOR_PALETTE

    float tileIndex = color.b * (span - 1.0);

    vec2 offset = GetOffsetInTile(color.rg);

    vec3 newColor0 = GetColor2D(floor(tileIndex), offset);
    vec3 newColor1 = GetColor2D(ceil(tileIndex), offset);

    vec3 newColor = mix(newColor0, newColor1, fract(tileIndex));
#endif // SAMPLER_3D_TEXTURE_COLOR_PALETTE

    return mix(color, newColor, weight);
}

void main()
{
    float meanVar = texture(sColorVariance, vTexCoordI).r;

    vec3 meanColor = texture(sColorMean, vTexCoordI).rgb;
    vec3 iColor = texture(sImage, vTexCoord[0]).rgb;
    float blurAlpha = uBlendSharpenLighten.x;
    float sharpenAlpha = uBlendSharpenLighten.y;
    float lightenFactor = uBlendSharpenLighten.z;

    float p = clamp((min(iColor.r, meanColor.r - 0.1) - 0.2) * 4.0, 0.0, 1.0);
    float kMin = clamp((0.11 / (meanVar + 0.1) - meanVar * 0.4) * p * blurAlpha, 0.0, 1.0);
    vec3 blend_color = mix(iColor, meanColor, kMin);

    vec3 sharpen_diff = iColor * uSharpenWeightArray[0];
    for (int i = 1; i < SHARPEN_TAP_COUNT; ++i) {
      sharpen_diff += texture(sImage, vTexCoord[i]).rgb * uSharpenWeightArray[i];
    }
    vec3 sharpen_color = clamp(blend_color + sharpen_diff, 0.0, 1.0);

    vec3 lighten_color = ColorPalette3D(sharpen_color, lightenFactor);

    cColor = vec4(lighten_color, 1.0);
}

#endif // RETROVK_FRAGMENT_SHADER
