#ifndef   TEXCOORDP
#define   TEXCOORDP highp
#endif // TEXCOORDP

#ifdef RETROVK_VERTEX_SHADER
#define varying out
#endif

#ifdef RETROVK_FRAGMENT_SHADER
#define varying in
#endif

#ifndef   KERNEL_HALF_PIXEL
#define   KERNEL_HALF_PIXEL 1
#endif // KERNEL_HALF_PIXEL

#ifndef   KERNEL_LENGTH
#define   KERNEL_LENGTH 9
#endif // KERNEL_LENGTH

#ifndef   KERNEL_STEP
#define   KERNEL_STEP 2
#endif // KERNEL_STEP

#ifndef   IS_EXTERNAL_TEXTURE
#define   IS_EXTERNAL_TEXTURE 0
#endif // IS_EXTERNAL_TEXTURE

#if       IS_EXTERNAL_TEXTURE
#extension GL_OES_EGL_image_external: enable
#extension GL_OES_EGL_image_external_essl3: enable
#define SAMPLER_TEXTURE_COLOR_IN samplerExternalOES
#else //  IS_EXTERNAL_TEXTURE
#define SAMPLER_TEXTURE_COLOR_IN sampler2D
#endif // IS_EXTERNAL_TEXTURE

varying TEXCOORDP vec2 vTexCoordH;
varying TEXCOORDP vec2 vTexCoordK;
flat varying TEXCOORDP vec2 vTexCoordOffset;
varying TEXCOORDP vec2 vTexCoord;


#ifdef RETROVK_VERTEX_SHADER

in vec4 aPosition;
in vec2 aTexCoord;

uniform vec2 uTexelOffset;

uniform mat4 uImageMatrix;

void main()
{
  gl_Position = aPosition;
  vTexCoord = (uImageMatrix * vec4(aTexCoord, 0, 1)).xy;

  vec2 texCoord = aTexCoord;
  vec2 singleStepOffset = uTexelOffset;

  vec2 single_step_offset = uTexelOffset;
  vTexCoordOffset = single_step_offset * float(KERNEL_STEP);

#if KERNEL_HALF_PIXEL
  float first_tap = -float(KERNEL_LENGTH * KERNEL_STEP) / 2.0;
  vTexCoordH = texCoord + single_step_offset * first_tap;
  float start = first_tap + 1.5;
#else  // KERNEL_HALF_PIXEL
  float start = -float((KERNEL_LENGTH - 1) * KERNEL_STEP) / 2.0;
#endif // KERNEL_HALF_PIXEL

  vTexCoordK = texCoord + single_step_offset * start;
}

#endif // RETROVK_VERTEX_SHADER

#ifdef RETROVK_FRAGMENT_SHADER

precision mediump float;

uniform SAMPLER_TEXTURE_COLOR_IN    sImage;
uniform sampler2D sColorMean;

layout(location = 0) out vec4 cVariance;
layout(location = 1) out vec4 cMean;

void main() {

#if KERNEL_HALF_PIXEL
  float sum_count = float(KERNEL_LENGTH) + 0.5;
  vec3 sum = texture(sColorMean, vTexCoordH).rgb / 2.0;
#else  // KERNEL_HALF_PIXEL
  float sum_count = float(KERNEL_LENGTH);
  vec3 sum = vec3(0);
#endif // KERNEL_HALF_PIXEL

  for (int i = 0; i < KERNEL_LENGTH; ++i) {
    sum += texture(sColorMean, vTexCoordK + float(i) * vTexCoordOffset).rgb;
  }

  vec3 meanColor = sum / sum_count;
  cMean = vec4(meanColor, 1.0);

  vec3 iColor = texture(sImage, vTexCoord).rgb;

  vec3 diffColor = (iColor - meanColor) * 7.07;
  diffColor = min(diffColor * diffColor, 1.0);

  float meanVar = (diffColor.r + diffColor.g + diffColor.b) / 3.0;

  cVariance.r = meanVar;
}

#endif // RETROVK_FRAGMENT_SHADER
